#ifndef VECTOR_H
#define VECTOR_H

#include "Tuple.h"

class Point;

class Vector : public Tuple<3>
{
public:
	float &x;
	float &y;
	float &z;

	Vector();
	Vector(const int i);
	Vector(const Vector &v);
	Vector(const Point &p);
	Vector(float x, float y, float z);

	Vector &operator=(const Vector &v);

	Vector operator+() const;
	Vector operator-() const;

	Vector &operator+=(const Vector &v);
	Vector &operator-=(const Vector &v);
	Vector &operator*=(float s);
	Vector &operator/=(float s);

	friend bool operator==(const Vector &u, const Vector &v);
	friend bool operator!=(const Vector &u, const Vector &v);
	friend bool operator>(const Vector &u, const Vector &v);   // Compare length
	friend bool operator<(const Vector &u, const Vector &v);   // Compare length

	friend Vector operator+(const Vector &u, const Vector &v);
	friend Vector operator-(const Vector &u, const Vector &v);
	friend float operator*(const Vector &u, const Vector &v);   // Dot product
	friend Vector operator*(float s, const Vector &v);
	friend Vector operator*(const Vector &v, float s);
	friend Vector operator/(const Vector &v, float s);
	friend float operator^(const Vector &v, const int n);
	friend float operator^(const Vector &u, const Vector &v);   // Angle between vectors
	friend Vector operator%(const Vector &u, const Vector &v);   // Cross product

	friend float N(const Vector &v);   // Norm
	friend float N2(const Vector &v);   // Squared norm

	float length() const;

	Vector &normalize();
};

#endif   // VECTOR_H
